/*
 * Created by wangshengye on 2020/05/07.
 */

#ifndef _SIGN_H
#define _SIGN_H

#ifdef __cplusplus
extern "C" {
#endif

enum BIO_TYPE
{
    BIO_FILE = 1,
    BIO_MEM
} _bio_type;

/*
 * @function:      数据签名函数
 * @input param:   date           待签名数据
 *                 data_size      待签名数据大小
 *                 pkey_file      私钥
 * @output_Param:  sig_buf,       sig_size签名后的数据和大小
 * @return:        0 正常，-1 签名失败
 */
int data_signature(const char* data, unsigned int date_size, unsigned char *sig_buf,
                    unsigned int *sig_size, const char *pkey_file);

/*
 * @function:      签名验证函数
 * @input param:   cert_file      用户证书,可以是证书的文件也可以是证书的内容字符串
 *                 bio_type       该参数传入值可以为BIO_FILE和BIO_MEM
 *                                当参数为BIO_FILE时表示cert_file是一个证书文件路径，为BIO_MEM时表示cert_file是证书的内容字符串
 *                 cert_size      当bio_type为BIO_MEM时该参数有效，表示证书内容字符串大小,当bio_type为BIO_FILE该参数无效可传0
 *                 org_data       原始数据
 *                 org_data_size  原始数据大小
 *                 sig_buf        签过名的数据
 *                 sig_size       sig_buf数据的大小
 * @output_Param:
 * @return:        0 正常, -1验证签名失败
 */
int data_verify(const char* cert_file, int bio_type, unsigned int cert_size,
                const char* org_data, unsigned int org_data_size, unsigned char *sig_buf, unsigned int sig_size);
/*
 * @function:      证书合法性验证
 * @input param:   user_cert      待验证的用户证书,可以是文件名也可以是保存证书的字符串
 *                 bio_type       证书保存类型.如果证书保存在文件中该参数使用BIO_FILE，如果保存在内存中则使用BIO_MEM
 *                 cert_size      当bio_type为BIO_MEM时该参数有效，表示证书内容字符串大小,当bio_type为BIO_FILE该参数无效可传0
 * @output_Param:
 * @return:        0 正常, -1证书验证失败
 */
int x509_cert_verify(const char* user_cert, int bio_type, unsigned int cert_size);

/*
 * @function:      检查证书与私钥是否匹配
 * @input param:   
 * @output_param:
 * @return:        不匹配返回NULL,匹配返回证书内容字符串，字符串保存在堆内存，使用结束需释放内存
 */

char* x509_check_private_key(void);

#ifdef __cplusplus
}
#endif

#endif //_SIGN_
